import sys
import subprocess
import gobject
import os
from os.path import join, getsize
import dbus
from dbus.mainloop.glib import DBusGMainLoop
import gnomevfs
import gettext
import re
import time

import __builtin__
__builtin__._ = gettext.gettext	


def free_space(dev):
	try:
        	stat = os.statvfs(dev)
	except:
        	# This could get called in the event loop as we're shutting down, after
	        # we've unmounted filesystems.
        	return 0
	return stat.f_bsize * stat.f_bavail

def calcular_tamano(dir):
	try:
		total = 0
		for root, dirs, files in os.walk(dir):
                	relpath = root.replace(dir,'')
			print relpath
			if re.search("^\/\.", relpath) != None:
				continue
			tam = 0
			for name in files:
                                if os.path.isfile(join(root, name)):
                                        tam = tam + getsize(join(root, name))
			total = total + tam
		return total
	except OSError, msg:
		print msg
		return -1

def popen(cmd):
	print >>sys.stderr, str(cmd)
	process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=sys.stderr, stdin=subprocess.PIPE)
	res = process.communicate()
	return process, res

def format_size(size):
	"""Format a partition size."""
	# Taken from ubiquity's ubiquity/misc.py
	if size == "":
		size = 0
	if size < 1024:
		unit = 'B'
		factor = 1
	elif size < 1024 * 1024:
		unit = 'kB'
		factor = 1024
	elif size < 1024 * 1024 * 1024:
		unit = 'MB'
		factor = 1024 * 1024
	elif size < 1024 * 1024 * 1024 * 1024:
		unit = 'GB'
		factor = 1024 * 1024 * 1024
	else:
		unit = 'TB'
		factor = 1024 * 1024 * 1024 * 1024
	return '%.1f %s' % (float(size) / factor, unit)

class formateador_Backend:
	def __init__(self, frontend):
		self.frontend = frontend
		self.devices = {}
		DBusGMainLoop(set_as_default=True)
		self.bus = dbus.Bus(dbus.Bus.TYPE_SYSTEM)
		hal_obj = self.bus.get_object('org.freedesktop.Hal', '/org/freedesktop/Hal/Manager')
		self.hal = dbus.Interface(hal_obj, 'org.freedesktop.Hal.Manager')

	def formatear_dispositivo(self, device):
		udi = self.hal.FindDeviceStringMatch('block.device', device)
		udi = udi[0]
		children = self.hal.FindDeviceStringMatch('info.parent', udi)
		for child in children:
			dev_obj = self.bus.get_object('org.freedesktop.Hal', child)
			child = dbus.Interface(dev_obj, 'org.freedesktop.Hal.Device')
			dev = str(child.GetProperty('block.device'))
			popen(['umount', dev])
		print str(device)
		popen(['umount', str(device)])
		res = popen(['parted', '-s', str(device), 'mklabel', 'msdos'])
		if res[0].returncode:
			print "Error al crear la tabla de particiones"
			return
		res = popen(['parted', '-s', str(device), 'mkpartfs', 'primary', 'fat32', '0', '--', '-0'])
		if res[0].returncode:
			print "Error al formatear el dispositivo"
		else:
			etiqueta = self.frontend.entry_etiquetado.get_text()
			if etiqueta != '':
				os.environ["MTOOLS_SKIP_CHECK"]="1"
				res = popen(['mlabel', '-i', dev, '::%s' % etiqueta])
			self.devices.pop(str(device))
			self.frontend.borrar_dispositivo(device)

	def detectar_dispositivos(self):
		devices = self.hal.FindDeviceByCapability('storage')
		for device in devices:
			dev_obj = self.bus.get_object('org.freedesktop.Hal', device)
			d = dbus.Interface(dev_obj, 'org.freedesktop.Hal.Device')
			if not self.es_disp_almacenamiento(d):
				continue
			
			# Comprueba si este dispositivo tiene un sistema de ficheros pero no tiene particiones
			if d.GetProperty('block.is_volume'):
				if d.GetProperty('volume.fstype') == 'vfat' or d.GetProperty('volume.fstype') == 'ext3':
					self.incluir_dispositivo(d)
			else: # En caso constrario, busca todas las particiones del dispositivo
				children = self.hal.FindDeviceStringMatch('info.parent', device)
				c = False
				devroot = d.GetProperty('block.device')
				for child in children:
					dev_obj = self.bus.get_object('org.freedesktop.Hal', child)
					child = dbus.Interface(dev_obj, 'org.freedesktop.Hal.Device')
					if child.GetProperty('block.is_volume'):
						if child.GetProperty('volume.fstype') == 'vfat' or child.GetProperty('volume.fstype') == 'ext3':
							c = True
							self.incluir_dispositivo(child)

				if not c and d.PropertyExists('storage.removable.media_size'):
					udi = d.GetProperty('info.udi')
					device = str(d.GetProperty('block.device'))
					self.incluir_dispositivo(d)
					self.devices[device] = {
					'label' : '',
					'fstype' : '',
					'uuid' : '',
					'mountpoint' : '',
					'udi' : udi,
					'free' : d.GetProperty('storage.removable.media_size'),
					'capacity' : d.GetProperty('storage.removable.media_size'),
					'device' : device,
					'devroot' : devroot
					}
					self.frontend.incluir_dispositivo(self.devices[device])
		self.bus.add_signal_receiver(self.dispositivo_detectado, "DeviceAdded", "org.freedesktop.Hal.Manager", "org.freedesktop.Hal", "/org/freedesktop/Hal/Manager")
		self.bus.add_signal_receiver(self.borrar_dispositivo, "DeviceRemoved", "org.freedesktop.Hal.Manager", "org.freedesktop.Hal", "/org/freedesktop/Hal/Manager")
	
	def incluir_dispositivo(self, dev):
		dev_obj = self.bus.get_object("org.freedesktop.Hal", dev.GetProperty('info.parent'))
		device = dbus.Interface(dev_obj, "org.freedesktop.Hal.Device")
		d = device.GetProperty('block.device')
		if self.devices.has_key(d):
			self.devices.pop(d)
			self.frontend.borrar_dispositivo(d)

		udi = dev.GetProperty('info.udi')
		mountpoint = str(dev.GetProperty('volume.mount_point'))
		device = str(dev.GetProperty('block.device'))
		self.devices[device] = {
			#'parent' : str(dev.GetProperty('info.parent')).replace('_', ' '),
			'label' : str(dev.GetProperty('volume.label')).replace(' ', '_'),
			'fstype' : str(dev.GetProperty('volume.fstype')),
			'uuid' : str(dev.GetProperty('volume.uuid')),
			'mountpoint' : mountpoint,
			'udi' : str(dev.GetProperty('info.udi')),
			'free' : mountpoint and free_space(mountpoint) or 0,
			'capacity' : dev.GetProperty('volume.size'),
			'device' : device,
			'devroot' : d
		}
		self.bus.add_signal_receiver(self.modificacion_estado, 'PropertyModified', 'org.freedesktop.Hal.Device', 'org.freedesktop.Hal', udi, path_keyword='udi')
		self.frontend.incluir_dispositivo(self.devices[device])

	def borrar_dispositivo(self, udi):
		d = None
		for device in self.devices.itervalues():
			if device['udi'] == udi:
				d = device['device']
				break
		if d:
			self.devices.pop(d)
		self.frontend.borrar_dispositivo(d)

	def dispositivo_detectado(self, udi):
		dev_obj = self.bus.get_object("org.freedesktop.Hal", udi)
		dev = dbus.Interface(dev_obj, "org.freedesktop.Hal.Device")
		
		success = False
		
		if dev.GetProperty('block.is_volume'):
			if dev.GetProperty('volume.fstype') == 'vfat' or dev.GetProperty('volume.fstype') == 'ext3':
				self.incluir_dispositivo(dev)	
		else: # En caso constrario, busca todas las particiones del dispositivo
			children = self.hal.FindDeviceStringMatch('info.parent', udi)
			c = False
			devroot = dev.GetProperty('block.device')
			for child in children:
				dev_obj = self.bus.get_object('org.freedesktop.Hal', child)
				child = dbus.Interface(dev_obj, 'org.freedesktop.Hal.Device')
				if child.GetProperty('block.is_volume'):
					if child.GetProperty('volume.fstype') == 'vfat' or child.GetProperty('volume.fstype') == 'ext3':
						c = True
						self.incluir_dispositivo(child)

			if not c and dev.PropertyExists('storage.removable.media_size'):
				udi = dev.GetProperty('info.udi')
				device = str(dev.GetProperty('block.device'))
				self.devices[device] = {
				'label' : '',
				'fstype' : '',
				'uuid' : '',
				'mountpoint' : '',
				'udi' : udi,
				'free' : dev.GetProperty('storage.removable.media_size'),
				'capacity' : dev.GetProperty('storage.removable.media_size'),
				'device' : device,
				'devroot' : devroot
				}
				#self.frontend.incluir_dispositivo(self.devices[device])
	
			print "APARATO 2-5: %s" % udi
		print "APARATO 3: %s" % udi

	def modificacion_estado(self, num_changes, change_list, udi):
		dev_obj = self.bus.get_object("org.freedesktop.Hal", udi)
		dev = dbus.Interface(dev_obj, "org.freedesktop.Hal.Device")
		mountpoint = str(dev.GetProperty('volume.mount_point'))
		device = str(dev.GetProperty('block.device'))
		devroot = self.devices[device]['devroot']
		d = dev.GetProperty('block.device')
		self.devices[device] = {
		'label' : str(dev.GetProperty('volume.label')).replace(' ', '_'),
		'fstype' : str(dev.GetProperty('volume.fstype')),
		'uuid' : str(dev.GetProperty('volume.uuid')),
		'mountpoint' : mountpoint,
		'udi' : str(dev.GetProperty('info.udi')),
		'free' : mountpoint and free_space(mountpoint) or 0,
		'capacity' : dev.GetProperty('volume.size'),
		'device' : device,
		'devroot' : devroot
		}
		self.frontend.actualizar_listado(self.devices[device])

	def es_disp_almacenamiento(self, d):
		if d.GetProperty('storage.bus') not in ['usb', 'mmc']:
			return False

		drive_types = ['disk',
                       'sd_mmc',
                       'smart_media',
                       'memory_stick',
                       'compact_flash']
		if d.GetProperty('storage.drive_type') in drive_types:
            		return True
